/**
    \file CThread.hpp
    \author Kurt Stege

    Thread Component.

    Mainly used by Cogen to generate composite components.
    May be used manually for any purpose.

    History:

    (ks) = Kurt Stege, kurt-stege@online.de

    2005-08-23 (ks):    File created.
    2005-08-23 (ks):    Current version.
*/

#ifndef NSigslot_CThread_hpp_INCLUDEGUARD
#define NSigslot_CThread_hpp_INCLUDEGUARD

#include "Sigslot/TSignature.hpp"

namespace NSigslot {

/*!
    CThread is a Sigslot component that implements a thread.
*/
class CThread
{
    public:

    CThread();
    ~CThread();

    /*!
        This slot returns immediately, and creates a thread that
        calls the signal of the component. See \c SIGNAL_thread()
        for details.
    */
    void SLOT_start ();

    /*!
        Each thread created by this 
    */
    NSigslot::TSignature0::CSignal SIGNAL_thread;

    NSigslot::TSignature1<bool>::CSignal INIT_active;
    void INIT_shutdown();

    private:

    static void start_mainloop (void *context);
    void private_mainloop();

    private:

    bool    m_shutdownIsActive;
    int     m_threadCount;
};

} // namespace NSigslot

#endif // NSigslot_CThread_hpp_INCLUDEGUARD

