/*!
    \file TJunctionBox.hpp
    \author Kurt Stege

    Implements a junction box that allows to store several
    receivers within one slot.

    Each junction box represents a node within a simple
    linked list ("einfach verkettete Liste) of the receiving
    slots.

    History:

    (ks) = Kurt Stege, kurt-stege@online.de

    2005-07-25 (ks):    File created.
    2005-07-25 (ks):    Current version.
*/


/*
    Note: The header file TSignature.hpp is not directly required.
    The template TJunctionBox just depends on the given template
    parameter that provides the signature. So, the caller of the
    TJunctionBox already has to include TSignature.hpp for itself.
    And when the user provides some other matching Signature, the
    TJunctionBox will work as well.
*/
// #include "Sigslot/TSignature.hpp"

#include "Sigslot/TStandardCables.hpp"


namespace NSigslot {

/*!
    TODO: Documentation for TJunctionBox.
*/

template <typename TemplateParameterSignature>
class TJunctionBox
{
    public:

    typedef TemplateParameterSignature tSignature;

    explicit TJunctionBox(typename tSignature::CSignal &sender, const typename tSignature::CSignal &receiver)
    :
        m_sig1(sender),
        m_sig2(receiver)
    {
        sender = NSigslot::TCableToMessageOperator<TSlot<tSignature, TJunctionBox> > (*this);
    }

    void operator()(const typename tSignature::CMessage &msg) const
    {
        m_sig1.emitPackedMessage (msg);
        m_sig2.emitPackedMessage (msg);
    }

    /*
        TODO: Provide some means to shut down the system.
        The created connection has to be deleted (e.g. by sender = CSender();)
        before the TJunctionBox object is deleted.
    */

    private:

    typename tSignature::CSignal m_sig1;
    typename tSignature::CSignal m_sig2;
};

} // namespace NSigslot
